#!/usr/bin/env python3

import sys,pandas as pd


reads_result = sys.argv[1]
asm_result = sys.argv[2]


res = [ i for i in [reads_result.strip(), asm_result.strip()] if i != "no_results"]
# print(res)
res = list(set(res))
# print(res)
if len(res) == 0:
    print("no_result")
    sys.exit(0)
else:
    print(':'.join(res))
    sys.exit(0)



#!/usr/bin/env python3


import pandas as pd
import sys
import numpy as np
import argparse


def _assess(args):
    
    qc = pd.read_csv(f"{args.qc}", sep = "\t")
    qc = qc.fillna("")
    
    qc = qc[(qc["Isolate"] == args.seqid) & (qc['Aln_outlier'].str.contains("below"))]
    if qc.empty:
        print("include")
    else:
        print("exclude")



def set_parsers():
    # setup the parser
    parser = argparse.ArgumentParser(description='Collate and write bohra report',formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('--qc',
        help='',
        default = '')
    parser.add_argument('--seqid',
        help='',
        default = '')
    parser.add_argument('--strict',
        help='',
        default = '')
    
    parser.set_defaults(func=_assess)
    args = parser.parse_args()
    
    if vars(args) == {}:
        parser.print_help(sys.stderr)
    else:
        # print(args)
        args.func(args)
	

def main():
    """
    run pipeline
    """

    args = set_parsers()
    

if __name__ == "__main__":
    main()








#!/usr/bin/env python3
import sys,  json
import pandas as pd


def get_genes(_file, _type, _dict):
    
    with open(_file, 'r') as f:
        data = f.read().strip().split('\n')
        header = data[0].split('\t')
        line = data[1].split('\t')
        for i in range(1,len(header)):
            # print(header[i])
            l = [gn for gn in line[i].split(',')] if _type != 'partials' else [f"{gn}^" for gn in line[i].split(',')]
            # l = line[i] if _type != 'partials' else f"{line[i]}^"
            # print(line[i])
            # print(l)
            if header[i] in _dict:
                    _dict[header[i]].extend(l)
            else:
                _dict[header[i]] = l
        print(_dict)
        return _dict

def extract_contigs_plasmids(plasmid: pd.DataFrame) -> list:
    
    contigs =[ f"{i.split()[0]}" for i in plasmid[plasmid["molecule_type"] == "plasmid"]['contig_id'].unique()]
    return contigs

def extract_genes_on_plasmid(amrfinder: pd.DataFrame, contigs: list) -> list:
    
    genes_on_plasmid = amrfinder[amrfinder["Contig id"].isin(contigs)]["Gene symbol"].unique()
    return genes_on_plasmid

def get_plasmid_designation_species(plasmid: pd.DataFrame, ctg:str) -> tuple:

    plasmid_designation = plasmid[plasmid["contig_id"].str.contains(ctg)]["mash_nearest_neighbor"].unique()[0]
    plasmid_species = plasmid[plasmid["contig_id"].str.contains(ctg)]["mash_neighbor_identification"].unique()[0]
    return plasmid_designation, plasmid_species

def make_amr_df(genes_dict: dict, genes_on_plasmid:list, plasmid:pd.DataFrame, amrout:pd.DataFrame) -> pd.DataFrame:
    
    for row in genes_dict:
        print(row)
        
        if row != 'Isolate':
            print(genes_dict[row])

            res = []
            for gene in genes_dict[row]:
                if gene in genes_on_plasmid:
                    ctg = amrout[amrout["Gene symbol"] == gene]["Contig id"].unique()[0]
                    plasmid_designation, plasmid_species = get_plasmid_designation_species(plasmid, ctg)
                    gene = f"{gene}[P]"
                
                res.append(gene)
            genes_dict[row] = ",".join(res)
    print(pd.DataFrame(genes_dict, index = [0]))
    return pd.DataFrame(genes_dict, index = [0])

def combine_plasmid_data(plasmid: pd.DataFrame, amrfinder: pd.DataFrame, contigs: list, sid:str) -> pd.DataFrame:
    
    data = {"Isolate": sid,
            "_children" : [],
            "mash_nearest_neighbor":"",
            "mash_neighbor_identification":"",
            "Present_on_contigs":"",
            "AMR_genes":"",
            "rep_type":""}
    plasmid = plasmid[plasmid["molecule_type"] == "plasmid"]
    
    mnm = []
    mni = []
    poc = []
    ag = []
    rt = []


    for p in plasmid["mash_nearest_neighbor"].unique():
        mnm.append(p)
       
        tmp = plasmid[plasmid["mash_nearest_neighbor"] == p]
        ctgs = [f"{i.split()[0]}" for i in tmp["contig_id"].unique()]
        mni.append(tmp["mash_neighbor_identification"].unique()[0])
        poc.append(",".join(ctgs))
        ag.append(",".join(sorted(amrfinder[amrfinder["Contig id"].isin(ctgs)]["Gene symbol"].unique())))
        rt.append(",".join(tmp["rep_type(s)"].unique()))
        child = {"Isolate": sid,}
        child["Plasmid accession"] = p
        child["Plasmid species"] = tmp["mash_neighbor_identification"].unique()[0]
        child["Present on contigs"] = ",".join([f"{i.split()[0]}" for i in tmp["contig_id"].unique()])
        child["Replicon type"] = ",".join(tmp["rep_type(s)"].unique())
        child["AMR genes"] = ",".join(sorted(amrfinder[amrfinder["Contig id"].isin(ctgs)]["Gene symbol"].unique()))
        data["_children"].append(child)
    data["Plasmid accession"] = "|".join(mnm) if mnm != [] else ""
    data["Plasmid species"] = "|".join(mni) if mni != [] else ""
    data["Present on contigs"] = "|".join(poc) if poc != [] else ""
    data["AMR genes"] = "|".join(ag) if ag != [] else ""
    data["Replicon type"] = "|".join(rt) if rt != [] else ""
    
    with open(f"plasmid.json", 'w') as f:
        json.dump(data, f, indent = 4)
    # return plasmid

_dict = {'Isolate': sys.argv[1]}

# matches
_dict = get_genes(_file = sys.argv[2], _type = 'matches', _dict = _dict)
# partials 
_dict = get_genes(_file = sys.argv[3], _type = 'partials', _dict = _dict)

amrfinder = pd.read_csv(sys.argv[4], sep = '\t')
plasmid = pd.read_csv(sys.argv[5], sep = '\t')

genes_on_plasmid = extract_genes_on_plasmid(amrfinder, extract_contigs_plasmids(plasmid))
contigs = extract_contigs_plasmids(plasmid)

combine_plasmid_data(plasmid, amrfinder, contigs, _dict['Isolate'])

df_amr = make_amr_df(_dict, genes_on_plasmid, plasmid, amrfinder)
df_amr.to_csv("resistome.txt", sep = '\t', index = False)


import pathlib
import datetime
import logging
import subprocess
from bohra.launcher.Utils import CustomFormatter, _check_path, _run_subprocess
from bohra.launcher.SetupInput import find_data
from bohra.launcher.Deps import dependencies
from bohra.launcher.BohraRun import _check_bohra_success, _run_bohra_cmd
# Logger
LOGGER =logging.getLogger(__name__) 
LOGGER.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
fh = logging.FileHandler('bohra.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter('[%(levelname)s:%(asctime)s] %(message)s', datefmt='%Y-%m-%d %I:%M:%S %p') 
fh.setFormatter(formatter)
LOGGER.addHandler(ch) 
LOGGER.addHandler(fh)


def _download_reads_from_github(isolate_list:list, download_stub:str):
    """
    Downloads test data from github.
    """
    LOGGER.info(f"Preparing to download test data.")
    LOGGER.info(f"Removing any existing test data directory.")
    _run_subprocess(cmd = "rm -rf test_data")
    LOGGER.info(f"Downloading test data from github.")
    for isolate in isolate_list:
        LOGGER.info(f"Downloading reads for {isolate}.")
        for r in [1,2]:
            cmd = f"mkdir -p test_data/{isolate} && wget --continue -O test_data/{isolate}/{isolate}_{r}.fastq.gz {download_stub}/{isolate}/{isolate}_{r}.fastq.gz"
            LOGGER.info(f"Running : {cmd}")
            _run_subprocess(cmd = cmd)
        
def _download_reference_from_github(download_stub:str):

    cmd = f"wget -O Lm_Cluster1_J1-108.fa {download_stub}/Lm_Cluster1_J1-108.fa"
    LOGGER.info(f"Downloading Lm_Cluster1_J1-108.fa to {pathlib.Path.cwd() / 'Lm_Cluster1_J1-108.fa'}")
    LOGGER.info(f"Running : {cmd}")
    _run_subprocess(cmd = cmd)
    return 'Lm_Cluster1_J1-108.fa'

def _check_reference_test(path, download_stub):

    if not pathlib.Path(path).exists():

        ref = _download_reference_from_github(download_stub=download_stub)
        if not pathlib.Path(ref).exists():
            LOGGER.critical(f"Reference could not be found or downloaded. Please check your internet connection and try again.")
            raise SystemExit

    return path

def _check_test_data(path, isolate_list):

    for i in isolate_list:
        reads = sorted(pathlib.Path(path).glob(f"{i}/{i}*.f*q.gz"))
        if len(reads) != 2:
            LOGGER.warning(f"Reads for {i} are not found. Will try to get them for you!")
            return False
    LOGGER.info(f"All reads are found at {path}")
    return True

def run_tests(cpus:int=1, shovill_ram:int=16, wdir:str=str(pathlib.Path.cwd())):
    # check dependencies first
    # if dependencies(_action = "check") == 0:
    download_stub = "https://raw.githubusercontent.com/MDU-PHL/bohra/master/data"
    read_path = f"{pathlib.Path.cwd() / 'test_data'}"
    isolate_list = ['ERR1102348','ERR1102353','ERR1102355','ERR1102356']
    # download and check reference
    reference = _check_reference_test(path=f"{pathlib.Path.cwd() / 'Lm_Cluster1_J1-108.fa'}", download_stub=download_stub)
    # download reads
    _download_reads_from_github(download_stub=download_stub, isolate_list=isolate_list)
    # if reads did not download correctly, raise exit
    if not _check_test_data(path = read_path, isolate_list = isolate_list):
        LOGGER.critical(f"Test data could not be found or downloaded. Please check your internet connection and try again.")
        raise SystemExit
    
    LOGGER.info(f"Reads have been downloaded to {read_path}.")
    LOGGER.info(f"Now generating the input file from the reads.")
    find_data(reads = f"{read_path}",contigs="",isolate_ids ="", outname="bohra_input.tsv" )
    # check that the input file has been created
    if not _check_path(path="bohra_input.tsv"):
        LOGGER.critical(f"Input file has not been created. Test failing. Exiting.")
        raise SystemExit
    # setup the report directory for running and testing
    report_outdir = f"bohra_test_output_{datetime.datetime.now().strftime('%Y%m%d_%H%M%S')}"
    
    cmd = f"bohra run full -i bohra_input.tsv -ref {reference} --cpus {cpus} --report_outdir {report_outdir} --workdir {pathlib.Path(wdir).resolve()} --shovill_ram {shovill_ram}"
    LOGGER.info(f"Running bohra with the command: {cmd}")
    proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
    for line in proc.stdout:
        print(line, end='')

    # Wait for the process to complete and get the return code
    proc.wait()
    
    # else:
    #     LOGGER.critical(f"Some bohra dependencies are missing or not installed properly. Please run 'bohra deps install' and try again.")
    #     raise SystemExit    



#!/usr/bin/env python3

import sys, json, pathlib
import pandas as pd





def extract_unclustered(df, col, col1):

    u = df.groupby(col)[col1].agg('count').reset_index()

    uc = list(u[u[col1] == 1][col])

    return uc

groups = sys.argv[1]


try:
    df = pd = pd.read_csv(groups, sep="\t")
    print(df)
    if "ST" in df.columns:
        cols = [i for i in df.columns if "Allele" in i]
        df["MLST_alleles"] = df[cols].apply(lambda x: ":".join(x), axis=1)
        col1 = df.columns[0]
        uc = extract_unclustered(df, col="MLST_alleles", col1=col1)

        df = df[~df["Isolate"].isin(uc)]

        df[["ID","MLST_alleles"]].to_csv("group.txt", sep="\t", index=False, header = False)
    else:
        cols = sorted([i.split(":")[-1] for i in df.columns if "Tx" in i])
        col1 = df.columns[0]
        df = df[df[col1] != "Reference"]
        
        uc = extract_unclustered(df, col=f"Tx:{cols[-1]}", col1=col1)
        df = df[df[f"Tx:{cols[-1]}"] != "UC"]

        df[["ID", f"Tx:{cols[-1]}"]].to_csv("group.txt", sep="\t", index=False, header = False)
except Exception as e:
    pd.DataFrame(columns=["ID","Group"]).to_csv("group.txt", sep="\t", index=False)


import click
import pathlib
import os

from bohra.launcher.SetupInput import find_data

@click.command()
@click.option('--reads',
              default="",
              help="Path to search for reads files, e.g. *.f*q.gz",)
@click.option('--contigs',
              default="",
              help="Path to search for assembly files, e.g. *.f*a.gz")
@click.option('--isolate_ids',
              default='',
              help="Path to a file containing at least one column 'Isolate' with isolate names. Optionally add 'species' and other columns you wish to use for further annotation of trees.")
@click.option('--outname',
              default='bohra_input.tsv',
              help="Name of the file to write the generated input table to.")


def generate_input(reads, contigs, isolate_ids, outname):
    """
    Generare input files for the Bohra pipeline.
    """
    
    if not reads and not contigs:
        click.echo("Please specify  --reads and/or --contigs.")
        return

    
    # click.echo(f"Generating input files for {input_type} in {path}.")

    find_data(
        isolate_ids=isolate_ids,
        reads=reads,
        contigs=contigs,
        outname=outname
    )

  